/************************************************************

  sources.c

  source terms for poscol simulation

************************************************************/
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "basics.h"
#undef pi
#include "MD_FCT_defs.h"
#include "MD_FCT.h"
#include "MD_FCT_utils.h"
#include "poscol.h"
#include "special.h"
#include "collisions.h"
#define pi 3.14159265
#define CORNER(i,j) (((i==0)||(i == (Nr+1)))&&((j==0)||(j==(Nu+1))))
#define TOP (Nr*axr_factor + (Nu+1)*axu_factor)


MD_FCT_species_ptr etasp, Grsp, Gzsp, nisp, Gammaisp, Ersp, Ezsp;
unsigned int axu_factor, axr_factor, rusize, u_infty, r_R, vr_offset, vu_offset;
geometry_ptr phase_space, configuration_space;
axis_ptr axr_config;
dblptr elastic = NULL, del_elastic = NULL, u_h = NULL, one_u_h = NULL;
dblptr eta_n, eta_np1, Gr_n, Gr_np1, Gz_n, Gz_np1, ne = NULL;
dblptr ionization = NULL;
double delel, kTi_e, kTi_M, sm_eps0, Vx, qi_M;
double dtkinetic, ion_sound_speed, dr_W, steom_3;
dblptr alpha = NULL, nuM = NULL;
dblptr v_sonic_u = NULL, v_sonic_r = NULL, u_av = NULL, phi = NULL, Gammae = NULL;
dblptr eta_TBC = NULL, Gr_TBC = NULL, Gz_TBC = NULL;

static dblptr C = NULL;
static dblptr t1 = NULL, f0 = NULL;
static double ion_elastic, twopisteom3;
static double Ez, Er_BC;
static unsigned int u_ctr = 0, u_freq = 16;
static double ni_W, Gammai_W, ni_BC, Gammai_BC;
double teoms, twopi_3teoms, BEAM_FACTOR = 1.0 ;
void print_row();
double get_X_gamma();
double get_Pterm();
double get_vterm();
/***************************************************

  print_neighborhood(y,pos)
  
  prints MxM neighborhood of 2-d array y

***************************************************/
void print_neighborhood(y,M)
dblptr y;
unsigned int M;
{
int i,j,rpos,upos,N;
unsigned int pos;
char outstr[100];

N = M/2;
pos = MD_get_position(phase_space);
for(j=N; j>=-N; j--){
  upos = axu->index + j;
  for(i=-N; i<=N; i++){
    rpos = (axr->index + i);
    if ((rpos > Nr+1)||(rpos < 0)||(upos > Nu+1)||(upos < 0)){
      sprintf(outstr, " ************ ");
    } else {
      sprintf(outstr," % 9.5e ",y[pos+i*axr_factor+j*axu_factor]);
    }
    fprintf(LOGFILE,"%s",outstr);
  }
  fprintf(LOGFILE,"\n");
}
return;
}
/************************************************************

  eta_OutFlow_u()

  Outflow boundary condition using u_char, v_sonic_u.

************************************************************/
void eta_OutFlow_u(bc)
MD_FCT_bc *bc;
{

  bc->vchar_r = 0.0;
  bc->Lchar_r = 0.0;
  bc->Sr = eta_TBC[axr->index]; bc->Vr = 0.0;
  bc->rho_amb_r = 0.0;
}
/************************************************************

  Gr_OutFlow_u()

  Outflow boundary condition using u_char, v_sonic_u.

************************************************************/
void Gr_OutFlow_u(bc)
MD_FCT_bc *bc;
{

  bc->Sr = Gr_TBC[axr->index]; 
  bc->Vr = 0.0;
  bc->vchar_r = 0.0;
  bc->Lchar_r = 0.0;
  bc->rho_amb_r = 0.0;
}
/************************************************************

  Gz_OutFlow_u()

  Outflow boundary condition using u_char, v_sonic_u.

************************************************************/
void Gz_OutFlow_u(bc)
MD_FCT_bc *bc;
{

  bc->Sr = Gz_TBC[axr->index]; 
  bc->Vr = 0.0;
  bc->vchar_r = 0.0;
  bc->Lchar_r = 0.0;
  bc->rho_amb_r = 0.0;
}
/************************************************************

  eta_wall()

  wall boundary condition for eta

************************************************************/
void eta_wall(bc)
MD_FCT_bc *bc;
{

  bc->Sr = SBC[axu->index];
  bc->Vr = 0.0;

  bc->Lchar_r = 0.0;
}
/************************************************************

  Gr_wall()

  wall boundary condition for Gr

************************************************************/
void Gr_wall(bc)
MD_FCT_bc *bc;
{
  bc->Sr = SBC[axu->index];
  bc->Vr = 0.0;

  bc->Lchar_r = 0.0;
}
/************************************************************

  Gz_wall()

  wall boundary condition for Gz

************************************************************/
void Gz_wall(bc)
MD_FCT_bc *bc;
{
  bc->Sr = SBC[axu->index];
  bc->Vr = 0.0;
  bc->Lchar_r = 0.0;
}
/************************************************************

  ni_wall()

  wall boundary condition for ni

************************************************************/
void ni_wall(bc)
MD_FCT_bc *bc;
{
  bc->Sr = ni_BC;
  bc->Vr = 0.0;
  bc->Lchar_r = 0.0;
}
/************************************************************

  Gammai_wall()

  wall boundary condition for ni

************************************************************/
void Gammai_wall(bc)
MD_FCT_bc *bc;
{
  bc->Sr = Gammai_BC;
  bc->Vr = 0.0;
  bc->Lchar_r = 0.0;
}
/************************************************************

  Er_wall()

  wall boundary condition for Er

************************************************************/
void Er_wall(bc)
MD_FCT_bc *bc;
{
  bc->Sr = Er_BC;
  bc->Vr = 0.0;
  bc->Lchar_r = 0.0;
}
/************************************************************

     get_X_gamma(X)

     gets X/gamma according to approximate inversion of
     the definition by the elliptic expansion.

************************************************************/
double get_X_gamma(X)
     double X;
{
  double result, Xs;

  Xs = X*X;
  if (Xs >= 1.0) {return(1.0);}
  result = (35.0+Xs*(92.0+Xs*17.0))/(105.0 + Xs*(24.0+Xs*15.0));
  return(result);
}
/************************************************************

     get_Pterm(X)

     gets (1 - X/gamma) according to approximate inversion of
     the definition by the elliptic expansion.

************************************************************/
double get_Pterm(X)
     double X;
{
  double result, Xs;

  Xs = X*X;
  if (Xs >= 1.0) {return(0.0);}
  result = ((70.0-Xs*(68.0+Xs*2.0)))/(105.0 + Xs*(24.0+Xs*15.0));
  return(result);
}
/************************************************************

     get_vterm(X)

     gets (3(X/gamma)-1)/(2X) according to approximate inversion of
     the definition by the elliptic expansion.

************************************************************/
double get_vterm(X)
     double X;
{
  double result, Xs;

  Xs = X*X;
  if (Xs >= 1.0) {return(1.0);}
  result = (126.0 + 18.0*Xs)/(105.0 + Xs*(24.0+Xs*15.0));
  return(result);
}

/************************************************************

  get_u_av()

  gets average energy based on eta->rho0[]

************************************************************/
void get_u_av()
{
  unsigned int i, j;
  unsigned int pos, posu1;

  for(i=1; i<=Nr; i++){
    axr->index = i;
    posu1 = i*axr_factor+axu_factor;
    for(j=1, pos = posu1; j<=Nu; j++, pos += axu_factor){
      t1[j] = etasp->rho0[pos]*axu->r[j];
    }
    u_av[i] = MD_integral_dV(t1, axu)/
      MD_integral_dVol(etasp->rho0, axu, phase_space);
  }
  u_av[0] = u_av[1];  u_av[Nr+1] = 0.0;
  return;
}
/************************************************************

  get_ion_wall_terms()

  gets ni_W, Gammai_W, ni_BC, Gammai_BC

************************************************************/
void get_ion_wall_terms(vr)
double vr;
{
  double d, x, c1, c2, E4, E4_, cs, x1, x2;

  d = (R - axr_config->r[Nr]);
  c1 = (vr+SQRT__PI_2*ion_sound_speed);
  x1 = (d*ion_elastic)/c1;
  E4 = (c1>0.0)?En(x1,4):0.0;
  c2 = (vr-SQRT__PI_2*ion_sound_speed);
  x2 = (d*ion_elastic)/c2;
  E4_ = (c2>0.0)?En(x2,4):0.0;
  ni_BC = Gammai_BC = 6.0*(E4 + E4_) - 1.0;
  cs = ion_sound_speed*ion_sound_speed;
  x = (E4 - E4_)/(sqrt_2*SQRT_PI*d*ion_sound_speed);
  ni_W = 1.5*(vr*vr+PI_2*cs)*x;
  Gammai_W = (vr*vr + 1.5*PI*cs)*x;
  return;
}
/************************************************************

  advance_sources()

************************************************************/
void advance_sources(set, dt)
double dt;
MD_FCT_species_set_ptr set;
{
dblptr ni_n, ni_np1, Gammai_n,  Gammai_np1;
dblptr Ez_n, Ez_np1, Er_n, Er_np1;
unsigned int i, j;
unsigned int pos, posu0, posu1;
double dt_eps0, dtkTi_M, dtsteom, dtqi_M;
double one_coll_G, one_coll_Gammai, one_coll_ni, one_coll_r, one_coll_z;
double Gammai_A, Gammai_B, G_eff;
double a, a_last, x,  t2;
static bool First_Time = true;
static char sw;
static dblptr one_coll = NULL;

if (First_Time) {
  one_coll = get_memory(one_coll, (Nu+1), double);
  C = get_memory(C, (Nr+2), double);
  sw = MD_Diffusion_re_use_Dkappa|MD_Diffusion_first_fractional_step;
  qi_M = qi/m_N;
  First_Time = false;
}
eta_n = etasp->rho_n;
eta_np1 = etasp->rho_np1;
Gr_n = Grsp->rho_n;
Gr_np1 = Grsp->rho_np1;
Gz_n = Gzsp->rho_n;
Gz_np1 = Gzsp->rho_np1;
ni_n = nisp->rho_n;
ni_np1 = nisp->rho_np1;
Gammai_n = Gammaisp->rho_n;
Gammai_np1 = Gammaisp->rho_np1;
Er_n = Ersp->rho_n;
Er_np1 = Ersp->rho_np1;
Ez_n = Ezsp->rho_n;
Ez_np1 = Ezsp->rho_np1;
dt_eps0 = dt/eps0;
dtkTi_M = dt*kTi_M; 
dtqi_M = dt*qi_M;
dtsteom = dt*steom;

/************************************************************
  
  Get diffusion solution for finite Ti...take eta_n -> eta_np1

************************************************************/
MD_Diffusion(etasp, dt, axu, Du, sw);
/************************************************************

  put together the inelastic transport matrix
  (1 - inelastic*dt + de_excitation_dat*dt)
  and solve it. Take eta_np1 -> eta_np1.

************************************************************/
Apply_inelastics(eta_np1, dt, false, true);
/************************************************************

  get parameters for ion wall collision rate
  get new ion concentrations from source terms

  include wall collision terms for Gr, Gz

  get C, D
  get Gr_np1
************************************************************/
for(i=1; i<=Nr; i++){
  posu0 = i*axr_factor;
  posu1 = posu0+axu_factor;
  axr_config->index = (axr->index = i);
  one_coll_ni = (i == Nr)?(1.0/(1.0+dt*ni_W)):1.0;
  ne[i] = twopisteom3*MD_integral_dVol(eta_np1, axu, phase_space);
  for(j=1, pos = posu1; j<=(Nu); j++, pos += axu_factor){
    axu->index = j;
    t1[j] = eta_np1[pos]*ionization[j];
  }
  ni_np1[i] = one_coll_ni*(ni_n[i]+dt*(
			   twopisteom3*MD_integral_dV(t1, axu)
			   ));
}
axr->index = Nr+1;
ne[Nr+1] = twopisteom3*MD_integral_dVol(eta_np1, axu, phase_space);
ne[0] = ne[1];
ni_np1[0] = nisp->bc[axr_config->sequence].Sl*ni_np1[1];
ni_np1[Nr+1] = ni_BC*ni_np1[Nr];
/*
  Get Ez, Er, Gr, Gammai
*/
for(axr->index=1; axr->index<=Nr+1; axr->index++){
  posu1 = axr->index*axr_factor + axu_factor;
  for(axu->index = 1, pos = posu1; axu->index <= Nu; axu->index++, pos += axu_factor){
    x = BEAM_FACTOR*axu->r[axu->index]*eta_np1[pos]*eta_np1[pos];
    x = x/(x+Gr_n[pos]*Gr_n[pos]+Gz_n[pos]*Gz_n[pos]);
    x = sqrt(x);
    Gz_np1[pos] = x*Gz_n[pos];
    Gr_np1[pos] = x*Gr_n[pos];
  }
  C[axr->index] = twopi_3teoms*MD_integral_dVol(Gz_np1, axu, phase_space);
  Gammae[axr->index] = twopi_3teoms*MD_integral_dVol(Gr_np1,axu, phase_space);  
}
I = -2.0*pi*q0*MD_integral_dV(C, axr_config);
(*Ez_np1) = ((*Ez_n) + dt*(I0 - I)/Cl);
one_coll_G = 1.0/(1.0 + dt*ion_elastic);
for(i=1; i<=Nr; i++){
  posu0 = i*axr_factor;
  posu1 = posu0+axu_factor;
  axr_config->index = (axr->index = i);

  one_coll_Gammai = (i == Nr)?(1.0/(1.0 + dt*(ion_elastic + Gammai_W))):one_coll_G;
  G_eff = Vx*(ni_np1[i]-ne[i])*axr->r[i]*axr->r[i]/(R*R);
  Er_np1[i] = (
		    Er_n[i] + dt_eps0*q0*(G_eff - (Gammai_n[i] - Gammae[i]))
		    )/
		 (1.0 - dt*Vx*2.0*axr->r[i]/(R*R));
  if (i == 1) {
    phi[0] = phi[1] = -.5*Er_np1[1]*axr->r[1];
  } else {
    phi[i] = phi[i-1] - .5*(Er_np1[i]+Er_np1[i-1])*(axr->r[i]-axr->r[i-1]);
  }
  Gammai_A = one_coll_Gammai*(Gammai_n[i] - dtkTi_M*MD_d_dr(ni_np1, axr_config, i));
  Gammai_B = one_coll_Gammai*dtqi_M*ni_np1[i];
  Gammai_np1[i] = Gammai_A + Er_np1[i]*Gammai_B;
  for(j=1, pos = posu1; j<=(Nu+1); j++, pos += axu_factor){
    f0[j] = eta_np1[pos]*one_u_h[j];
  }    
  f0[0] = f0[1]; 
  a_last = sqrt(f0[0]*f0[1]);
  for(j=1, pos = posu1; j<=(Nu); j++, pos += axu_factor){
    axu->index = j;
    if (i == 1) {
      one_coll[j] = 1.0/(1.0 + dt*nuM[j]);
    }
    one_coll_r = (i == Nr)?(1.0/(1.0+dt*(nuM[j]+Gr_W[j]))):one_coll[j];
    one_coll_z = (i == Nr)?(1.0/(1.0+dt*(nuM[j]+Gz_W[j]))):one_coll[j];

    a = sqrt(f0[j]*f0[j+1]);
    t2 = dtsteom*axu->r[j]*u_h[j]*2.0*(a - a_last)/(axu->r[j+1]-axu->r[j-1]);
    Gr_np1[pos] = one_coll_r*(
			       Gr_n[pos] - 
			       dtsteom*axu->r[j]*MD_d_dr(eta_np1, axr, pos) 
			       + Er_np1[i]*t2
			       );
    Gz_np1[pos] = one_coll_z*(Gz_n[pos] + (*Ez_np1)*t2);
    if (i == Nr) {
      Gr_np1[pos+axr_factor] = SBC[j]*Gr_np1[pos];
      Gz_np1[pos+axr_factor] = SBC[j]*Gz_np1[pos];
    }
    a_last = a;
  }
  Gz_np1[pos] = Gz_TBC[i]*Gz_np1[pos-axu_factor];
  Gr_np1[pos] = Gr_TBC[i]*Gr_np1[pos-axu_factor];
  Gz_np1[posu0] = 0.0;
}
axr->index = Nr+1;
/*
  boundary conditions
*/
Er_np1[Nr+1] = (Er_BC = (axr_config->r[Nr]/axr_config->r[Nr+1]))*Er_np1[Nr];
phi[Nr+1] = phi[Nr] - .5*(Er_np1[Nr+1]+Er_np1[Nr])*(axr->r[Nr+1]-axr->r[Nr]);
Gammai_np1[0] = Gammaisp->bc[axr_config->sequence].Sl*Gammai_np1[1];
Gammai_np1[Nr+1] = Gammai_BC*Gammai_np1[Nr];
Gammae[0] = -Gammae[1];
/* fix up corners */
Gr_np1[0] = Gr_np1[r_R] = 0.0;
Gr_np1[u_infty] = -Gr_TBC[1]*Gr_np1[u_infty+axr_factor-axu_factor];
Gr_np1[rusize-1] = SBC[Nu+1]*Gr_TBC[Nr]*Gr_np1[rusize-1-axr_factor-axu_factor];
Gz_np1[0] = Gz_np1[r_R] = 0.0;
Gz_np1[u_infty] = Gz_np1[u_infty+axr_factor];
Gz_np1[rusize-1] = SBC[Nu+1]*Gz_np1[rusize-1-axr_factor];
/*
  get average energy every u_freq steps
*/
if (MD_FCT_step == half){
  if (!u_ctr){ get_u_av();}
  u_ctr = (u_ctr+1)%u_freq;
}
return;
}
/************************************************************

  sources()

************************************************************/
void sources(set, dt)
MD_FCT_species_set_ptr set;
double dt;
{
static double Mi_qi, sMi_qi;
double vu, maxE; 
double absEr, maxEr_dr, maxEr, smaxEr_dr, vr, vz;
double b, c, d, s, x, r, du, eta_, g1_r, g1_z;
dblptr eta, Gr, Gz, ni, Gammai, Er;
unsigned int i, j, pos, vu_pos, vr_pos;
static unsigned int pnt;
static bool First_Time = true;

if (First_Time){
  MD_LOGFILE = LOGFILE;
  MD_Courant_retry_factor = .7;
  MD_Courant_update_factor = .9;
  MD_Courant = .25;
  etasp = MD_get_spptr("eta",set);  
  Grsp = MD_get_spptr("Gr",set);  
  Gzsp = MD_get_spptr("Gz",set);  
  phase_space = etasp->grid;
  axu_factor = axu->mul;
  axr_factor = axr->mul;
  r_R = (Nr+1)*axr_factor;
  rusize = (Nr+2)*(Nu+2);
  u_infty = (Nu+1)*axu_factor;
  vu_offset = axu->sequence*rusize;
  vr_offset = axr->sequence*rusize;
  ne = get_memory(ne, (Nr+2), double);
  v_sonic_u = get_memory(v_sonic_u, (Nr+2), double);
  v_sonic_r = get_memory(v_sonic_r, (Nu+2), double);
  eta_TBC = get_memory(eta_TBC, (Nr+2), double);
  Gr_TBC = get_memory(Gr_TBC, (Nr+2), double);
  Gz_TBC = get_memory(Gz_TBC, (Nr+2), double);
  f0 = get_memory(f0, (Nu+3), double);
  t1 = get_memory(t1, (Nu+2), double);

  nisp = MD_get_spptr("ni",set);
  configuration_space = nisp->grid;
  axr_config = MD_get_axptr("r", configuration_space);
  Gammaisp = MD_get_spptr("Gammai",set);
  Ezsp = MD_get_spptr("Ez",set);

  Mi_qi = m_N/qi;
  sMi_qi = sqrt(Mi_qi);
  steom_3 = steom/3.0;
  sm_eps0 = sqrt(m_elec/eps0);
  teoms = teom*teom;
  twopi_3teoms = (2.0*pi/3.0)*teoms;
  twopisteom3 = 2.0*pi*(steom*steom*steom);
  ion_elastic = qi/(m_N*mui);
  ion_sound_speed = sqrt((2.0*k*Ti)/m_N);
  free(Gammaisp->vel);
  free(Gzsp->vel); 
  Gzsp->vel = Grsp->vel;

  Gammaisp->vel = nisp->vel;
  u_av = get_memory(u_av, (Nr+2), double);
  Gammae = get_memory(Gammae, (Nr+2), double);
  phi = get_memory(phi, (Nr+2), double);
/*
  get any u-dependent arrays
*/
  delel = 2*m_elec/m_N;
  kTi_e = k*Ti/q0;
  kTi_M = k*Ti/m_N;
  get_elastic();
  get_inelastics();
  get_step_control();
  get_u_av();
  f0[Nu+2] = 0.0;
  Vx = .05*v_sonic_r[Nu];
  pnt = 10;
  First_Time = false;
}
/*
  Calculate velocities based on sp->rho
*/
eta = etasp->rho;
Gr = Grsp->rho;
Gz = Gzsp->rho;
ni = nisp->rho;
Gammai = Gammaisp->rho;
Er = Ersp->rho;
Ez = Ezsp->rho[0];

for(axr->index=i=1; i<= (Nr); axr->index = ++i){
  for(axu->index=j=0; j<= (Nu+1); axu->index = ++j){
    pos = MD_get_position(phase_space);
    g1_r = Gr[pos]; g1_z = Gz[pos]; eta_ = eta[pos];
    s = (eta_ != 0.0)?(steom_3/eta_):0.0;
/* Forcing vr, vu to be well-behaved */
    x = BEAM_FACTOR*axu->r[j]*eta_*eta_;
    x = (j)?x/(x+g1_r*g1_r+g1_z*g1_z):0.0;
    x = sqrt(x);
    s *= x;
    vr = g1_r*s;
    vz = g1_z*s;
    vu_pos = vu_offset + pos;
    vr_pos = vr_offset + pos;
    vu = -vr*Er[i] - Ez*vz;
    etasp->vel[vu_pos] = vu-del_elastic[j];
    etasp->vel[vr_pos] = vr;
    if (i == 1) {
      etasp->vel[vu_pos - axr_factor] = etasp->vel[vu_pos];  
      etasp->vel[vr_pos - axr_factor] = -etasp->vel[vr_pos];
    }
    if (i == Nr){
      etasp->vel[vu_pos + axr_factor] = etasp->vel[vu_pos];
      etasp->vel[vr_pos + axr_factor] = etasp->vel[vr_pos];
    } 
  }
  v_sonic_u[i] = v_sonic_r[Nu]*ABS(Ez);
  du = axu->r[Nu+1] - axu->r[Nu];
  pos = i*axr_factor + Nu*axu_factor;
  x = eta[pos]/eta[pos-pnt*axu_factor];
  r = du/(axu->r[Nu] - axu->r[Nu-pnt]);
 Gz_TBC[i] = Gr_TBC[i] = eta_TBC[i] = pow(x,r);
}
for(i=1; i<= (Nr+1); i++){
  nisp->vel[i] = (ni[i] != 0.0)?(Gammai[i]/ni[i]):0.0;
}
nisp->vel[0] = -nisp->vel[1]; 
/*
  get ion wall collision terms
*/
get_ion_wall_terms(nisp->vel[Nr]);
/*
  Advance variables by updating source terms
*/
MD_FCT_Richardson(set, advance_sources, dt);

for(i=1; i<= (Nr+1); i++){
  Ersp->vel[i] = Vx*axr->r[i]*axr->r[i]/(R*R);
}
Ersp->vel[0] = Ersp->vel[1];
/*
  Determine new time step

  get maximum Er, max Er/dr;
  then scan through u to implement approximate
  time step control; then do ion drift condition;

*/
MD_Source_dt = dtkinetic;
maxEr = 0.0; maxEr_dr = 0.0;
for(i=1; i<=Nr; i++){
  absEr = ABS(Er[i]);
  maxEr = max(maxEr, absEr);
  maxEr_dr = max(maxEr_dr, absEr/(axr_config->r[i+1] - axr_config->r[i])); 
}
maxE = maxEr + ABS(Ez);
smaxEr_dr = sqrt(maxEr_dr);
for(j=1; j<=Nu; j++){
  c = nuM[j];
  d = alpha[j]*maxE*maxE;
  b = (c+sqrt(c*c+2.0*d))/d;
  MD_Source_dt = min(MD_Source_dt, b);
}
MD_Source_dt = min(MD_Source_dt, ((sMi_qi/smaxEr_dr)*
		   max(1.0, (1.0/(mui*sMi_qi*smaxEr_dr)))));
/*
  print status and log data
*/
if (MD_FCT_step == half){
  fprintf(stderr,"MD_Source_dt: %8.5e total time: %8.5e ",MD_Source_dt, MD_Total_time);
  fprintf(stderr,"Ez: %8.5e I: %8.5e\r", Ez,I);
}
return;
}

